*******************************************************************************
*   File........: afill3d.prg
*   Date........: September 30, 1993
*   Author......: Ly Minh Tr (CIS 70641,3127)
*   Copyright...: None...freeware!
*   Version.....: 1.02
*******************************************************************************
*   PURPOSE:    Initializes a given array to the given value.
*
*   SYNTAX:     afill3d(<parray> [, <pexp> [, <nplane> [, <naxis>]]])
*
*   PARAMETERS: <parray>    = the 3D-array to store <pexp> to
*               <pexp>      = value to store in array, default is .F.
*               <nplane>    = plane to fill <pexp> with
*               <naxis>     = 0 or empty to delete plane on x-axis
*                           = 1 to delete plane y-axis
*                           = 2 to delete plane z-axis
*
*   RETURNS:    Boolean		= .T. if successful, .F. if init. fails
*
*   SEE ALSO:   acount3d(), adel3d(), aelem3d(), adim3d(), ains3d(), alen3d(),
*               aloc3d(), astore3d(), asubs3d()
*******************************************************************************
PARAMETERS parray, pexp, nplane, naxis
PRIVATE lval, nlen, nrow, ncol, nwidth
PRIVATE nloc, i, j, k
EXTERNAL ARRAY parray

lval   = .F.
nplane = IIF(EMPTY(nplane), 0, nplane)
naxis  = IIF(EMPTY(naxis), 0, naxis)

DO CASE
    CASE !INLIST(PARAMETER(), 1, 2, 3, 4)
        WAIT WINDOW "Invalid number of parameters given!" TIMEOUT 1

    CASE TYPE("parray[1]") <> "C" OR !([3D] $ parray[1])
        WAIT WINDOW "Given array is not a 3D-array!"

    CASE TYPE("nplane") <> "N"
        WAIT WINDOW "Invalid plane subscript!" TIMEOUT 1

    CASE !INLIST(naxis, 0, 1, 2)
        WAIT WINDOW "Invalid axis specifier!" TIMEOUT 1

    OTHERWISE
        nrow    = INT(VAL(SUBSTR(parray[1], 4)))
        ncol    = INT(VAL(SUBSTR(parray[1], AT(",", parray[1], 2)+1)))
        nwidth  = INT(VAL(SUBSTR(parray[1], AT(",", parray[1], 3)+1)))
        lval    = .T.
ENDCASE

IF lval
    DO CASE
        CASE nplane == 0
            nlen = ALEN(parray)
            FOR i = 2 TO nlen
                parray[i] = pexp
            ENDFOR

        CASE naxis == 0
            IF !BETWEEN(nplane, 1, nrow)
                WAIT WINDOW [Invalid x-axis plane subscript given!] TIMEOUT 1
                lval = .F.
            ELSE
                nloc = aloc3d(@parray, nplane, 1, 1)
                nlen = nloc + (ncol * nwidth) - 1

                FOR i = nloc TO nlen
                    parray[i] = pexp
                ENDFOR
            ENDIF

        CASE naxis == 1
            IF !BETWEEN(nplane, 1, ncol)
                WAIT WINDOW [Invalid y-axis plane subscript given!] TIMEOUT 1
                lval = .F.
            ELSE
                FOR i = 1 TO nrow
                    nloc = aloc3d(@parray, i, nplane, 1)
                    nlen = nloc + nwidth - 1

                    FOR j = nloc TO nlen
                        parray[j] = pexp
                    ENDFOR
                ENDFOR
            ENDIF

        CASE naxis == 2
            IF !BETWEEN(nplane, 1, nwidth)
                WAIT WINDOW [Invalid z-axis plane subscript given!] TIMEOUT 1
                lval = .F.
            ELSE
                FOR i = 1 TO nrow
                    FOR j = 1 TO ncol
                        nloc = aloc3d(@parray, i, j, nplane)
                        parray[nloc] = pexp
                    ENDFOR
                ENDFOR
            ENDIF
    ENDCASE
ENDIF

RETURN lval
