*******************************************************************************
*   File........: aloc3d.prg
*   Date........: September 30, 1993
*   Author......: Ly Minh Tr (CIS 70641,3127)
*   Copyright...: None...freeware!
*   Version.....: 1.02
*******************************************************************************
*   PURPOSE:    aloc3d() returns the location of the element belonging to
*               a 3D-array that is represented linearly.  Four parameters
*               are required.  The first one is the name of the 3D-array,
*               while the last 3 are the subscripts of the desired element.
*
*   SYNTAX:     aloc3d(<parray>, <i>, <j>, <k>)
*
*   PARAMETERS: <parray>    = the 3D-array to be indexed
*               <i>         = subscript for dimension 1
*               <j>         = subscript for dimension 2
*               <k>         = subscript for dimension 3
*
*   RETURNS:    Numeric
*
*   SEE ALSO:   acount3d(), adel3d(), aelem3d(), adim3d(), afill3d(), ains3d(),
*               alen3d(), astore3d(), asubs3d()
*******************************************************************************
PARAMETER parray, i, j, k
PRIVATE nindex, nrow, ncol, nwidth
EXTERNAL ARRAY parray

IF PARAMETER() <> 4
    WAIT WINDOW "Invalid number of parameters given!" TIMEOUT 1
    RETURN 0
ENDIF

*
* First element of a 3D-array contains the array ID and hold the dimensions
* of the array.  If this is not true, then the array is not a 3D-array!
*
IF TYPE("parray[1]") <> "C" OR !([3D] $ parray[1])
    WAIT WINDOW "Given array is not a 3D-array!" TIMEOUT 1
    RETURN 0
ENDIF                                                 

nrow    = INT(VAL(SUBSTR(parray[1], 4)))
ncol    = INT(VAL(SUBSTR(parray[1], AT(",", parray[1], 2)+1)))
nwidth  = INT(VAL(SUBSTR(parray[1], AT(",", parray[1], 3)+1)))

DO CASE
    CASE (TYPE("i") <> "N") OR !BETWEEN(i, 1, nrow)
        WAIT WINDOW "Invalid row subscript value!" TIMEOUT 1
        nindex = 0

    CASE (TYPE("j") <> "N") OR !BETWEEN(j, 1, ncol)
        WAIT WINDOW "Invalid column subscript value!" TIMEOUT 1
        nindex = 0

    CASE (TYPE("k") <> "N") OR !BETWEEN(k, 1, nwidth)
        WAIT WINDOW "Invalid width subscript value!" TIMEOUT 1
        nindex = 0

    OTHERWISE
        *
        * Offset by 1 more (at end of formula) to skip the dimensions
        * part at start of array
        *
        nindex = 1 + ((i - 1) * ncol + (j - 1)) * nwidth + (k - 1) + 1
ENDCASE

RETURN nindex
