*******************************************************************************
*   File........: asubs3d.prg
*   Date........: September 30, 1993
*   Author......: Ly Minh Tr (CIS 70641,3127)
*   Copyright...: None...freeware!
*   Version.....: 1.02
*******************************************************************************
*   PURPOSE:    Returns an element's row, column or width subscript for a
*               3D-array.
*
*   SYNTAX:     asubs3d(<parray>, <nelem>, <naxis>)
*
*   PARAMETERS: <parray>    = the 3D-array
*               <nelem>     = the element number in the array
*               <naxis>     = 1 for x-axis (row)
*                           = 2 for y-axis (column)
*                           = 3 for z-axis (width)
*
*   RETURNS:    Numeric     = subscript if successful, 0 if fails
*
*   REMARKS:    Mimics Foxpro's ASUBSCRIPT() for a 3D-array.
*
*   SEE ALSO:   acount3d(), adel3d(), aelem3d(), adim3d(), afill3d(), ains3d(),
*               alen3d(), aloc3d(), astore3d()
*******************************************************************************
PARAMETER parray, nelem, naxis
PRIVATE nval, nlen, nrow, ncol, nwidth, nr, nc, nw
EXTERNAL ARRAY parray

IF PARAMETER() <> 3
    WAIT WINDOW [Invalid number of parameters given!] TIMEOUT 1
    RETURN 0
ENDIF

*
* First element of a 3D-array contains the array ID and hold the dimensions
* of the array.  If this is not true, then the array is not a 3D-array!
*
IF TYPE("parray[1]") <> "C" OR !([3D] $ parray[1])
    WAIT WINDOW "Given array is not a 3D-array!" TIMEOUT 1
    RETURN 0
ENDIF

nval = 0
nlen = alen3d(@parray) + 1          && Add 1 to offset for array header

DO CASE
    CASE (TYPE("nelem") <> "N") OR !BETWEEN(nelem, 1, nlen)
        WAIT WINDOW "Invalid element value!" TIMEOUT 1

    CASE (TYPE("naxis") <> "N") OR !BETWEEN(naxis, 1, 3)
        WAIT WINDOW "Invalid dimension specifier!" TIMEOUT 1

    OTHERWISE
        nelem   = nelem - 1           && Subtract 1 to offset for array header

        nrow    = INT(VAL(SUBSTR(parray[1], 4)))
        ncol    = INT(VAL(SUBSTR(parray[1], AT(",", parray[1], 2)+1)))
        nwidth  = INT(VAL(SUBSTR(parray[1], AT(",", parray[1], 3)+1)))

        *
        * Calculate the corresponding subscript value
        *
        DO CASE
            CASE naxis = 1                              && Row or x-axis
                IF (nelem % (ncol * nwidth)) = 0
                    nr = INT(nelem / (ncol * nwidth))
                ELSE
                    nr = INT(nelem / (ncol * nwidth)) + 1
                ENDIF
                nval = nr

            CASE naxis = 2                              && Column or y-axis
                IF (nelem % (ncol * nwidth)) = 0
                    nr = INT(nelem / (ncol * nwidth))
                ELSE
                    nr = INT(nelem / (ncol * nwidth)) + 1
                ENDIF

                nc = nelem - (nr - 1) * (ncol * nwidth)
                IF (nc % nwidth) = 0
                    nc = INT(nc / nwidth)
                ELSE
                    nc = INT(nc / nwidth) + 1
                ENDIF
                nval = nc

            CASE naxis = 3                              && Width or z-axis
                IF (nelem % nwidth) = 0
                    nw = nwidth
                ELSE
                    nw = (nelem % nwidth)
                ENDIF
                nval = nw
        ENDCASE
ENDCASE

RETURN nval
